#!/bin/sh
# ------------------------------------------------------------------------
# This file:     /usr/local/nagios/sbin/InterfaceTableReset.cgi
#
# Permissions:   
#
# # ls -la /usr/local/nagios/sbin/InterfaceTableReset.cgi
# -rwxr-xr-x 1 nagios nagios 2522 Nov 16 13:14 /usr/local/nagio<...>
#
# Purpose:
#
# Simple cgi and shell script to remove a file from /tmp/.ifState in 
# a secure manner
#
# URL example (should be one line): 
#
# http://nagios.itdesign.at/nagios/cgi-bin/InterfaceTableReset.cgi?Comma
# nd=rm&What=/tmp/.ifState/server1-Interfacetable.txt
#
# This script is called recursively when executed as cgi script to 
# remove the tmp file. The unix "rm" command must have the correct 
# privileges to do that.
#
# Add the local web servers account to the /etc/sudoers file with
# "visudo".
#
# Result under SUSE Linux Enterprise Server 10:
#
# # grep InterfaceTableReset /etc/sudoers
# wwwrun ALL=(ALL) NOPASSWD: /usr/local/nagios/sbin/InterfaceTableReset.cgi
#
# ------------------------------------------------------------------------

# ------------------------------------------------------------------------
# Please add the complete filename of the script here
# This must be the path where your local web server calls this cgi-script
#
# This default is for a compiled nagios installation
# ------------------------------------------------------------------------
ThisScript="/usr/local/nagios/sbin/InterfaceTableReset.cgi"

# ------------------------------------------------------------------------
# Specify the State directory where the interface tables are stored
#
# This is the default directory. Please read the online documentation
# from the check_interface_table.pl plugins for details
# ------------------------------------------------------------------------
Directory="/tmp/.ifState";

# ------------------------------------------------------------------------
# if this script is called (recursively) as shell script 
# remove the tmp file and exit
# ------------------------------------------------------------------------
if [ "X$1" == "Xremove" ]; then
    if [ "X$2" != "X" -a "X$Directory" != "X" ]; then
        rm -f $Directory/$2
        exit $?
    fi
    exit 4
fi

# ------------------------------------------------------------------------
# if this script is called as cgi script 
# check query string and try to remove the file with "sudo"
# ------------------------------------------------------------------------

# CGI header
echo "Content-Type: text/html; charset=ISO-8859-1"
echo

Command=`echo $QUERY_STRING | awk -F\& '{print $1}'`
What=`echo $QUERY_STRING | awk -F\& '{print $2}'`

if [ "X$Command" == "XCommand=rm" ]; then
    FullFileName=`echo $What | cut -d= -f2`

    DirName=`dirname $FullFileName 2>/dev/null`
    BaseName=`basename $FullFileName 2>/dev/null`

    # check if file is in a valid directory
    if [ "X$DirName" == "X$Directory" ]; then
        # call this script again and try to remove the
        # file with "sudo"
        sudo $ThisScript remove $BaseName
        if [ $? -eq 0 ]; then
            echo done
            exit 0
        else
            echo remove failed
            exit 1
        fi
    else 
        echo wrong directory
        exit 2
    fi
fi
echo strange error
exit 3

